import xarray as xr
import numpy as np
from typing import Callable
from pathlib import Path
from strataer.utils import regridding_weights, planck, linear_weight


BASE_FILENAME = "input4MIPs_aerosolProperties_CMIP_UOEXETER-CMIP-1-3-0_gnz_175001-202312.nc"
clim_filename = "input4MIPs_aerosolProperties_CMIP_UOEXETER-CMIP-1-3-0_gnz_185001-202112-clim.nc"


def planck_weights(
    wavelength_1: float | np.ndarray, wavelength_2: float | np.ndarray, temperature: float
) -> float | np.ndarray:
    """
    compute the area under of the plank curve between wavelength_1 and wavelength_2.
    If inputs are numpy arrays they should be of the same size.
    """
    if isinstance(wavelength_2, np.ndarray) and isinstance(wavelength_1, np.ndarray):
        weights = []
        for w1, w2 in zip(wavelength_1, wavelength_2):
            wavel = np.sort(np.linspace(w1, w2, 1000).flatten())
            weights.append(np.trapezoid(planck(wavel, temperature), wavel))
        return np.array(weights)
    else:
        wavel = np.sort(np.linspace(wavelength_1, wavelength_2, 1000).flatten())
        return np.trapezoid(planck(wavel, temperature), wavel)


def solar_planck_weights(wavelength_1: float | np.ndarray, wavelength_2: float | np.ndarray):
    return planck_weights(wavelength_1=wavelength_1, wavelength_2=wavelength_2, temperature=5777.0)


def terrestrial_planck_weights(wavelength_1: float | np.ndarray, wavelength_2: float | np.ndarray):
    return planck_weights(wavelength_1=wavelength_1, wavelength_2=wavelength_2, temperature=254.356)


def wavel_interpolation_weights(
    input_wavel_bnds: xr.DataArray,
    output_wavel_bnds: xr.DataArray,
    input_wavel: xr.DataArray = None,
    output_wavel: xr.DataArray = None,
    weight_function: Callable = solar_planck_weights,
) -> xr.DataArray:
    """
    Determine the wavelength interpolation matrix given an input and output grid.

    Parameters
    ----------
    input_wavel_bnds : xr.DataArray
        wavelength bounds of input grid
    output_wavel_bnds : xr.DataArray
        wavelength bounds of output grid
    input_wavel : xr.DataArray, optional
        midpoints of input grid, by default None and will be calculated from the bounds.
    output_wavel : xr.DataArray, optional
        midpoints of output grid, by default None and will be calculated from the bounds.

    Returns
    -------
    xr.DataArray
        2d matrix of weights
    """

    input_bounds = input_wavel_bnds.values
    output_bounds = output_wavel_bnds.values
    weights = regridding_weights(input_bounds, output_bounds, weight_function=weight_function)

    if input_wavel is None:
        input_wavel = input_bounds[0:-1] + np.diff(input_bounds) / 2

    if output_wavel is None:
        output_wavel = output_bounds[0:-1] + np.diff(output_bounds) / 2

    return xr.DataArray(
        weights,
        dims=["new_wavelength", "wavelength"],
        coords=[output_wavel, input_wavel],
    )


def interpolate_variable(
    data: xr.Dataset,
    input_wavel_bnds: xr.DataArray,
    output_wavel_bnds: xr.DataArray,
    weight_function: Callable = linear_weight,
    weights: xr.DataArray = None,
) -> xr.Dataset:
    """
    Interpolate a variable to a new spectral grid with optional weights.

    Parameters
    ----------
    data : xr.DataArray
        data to be regridded
    input_wavel_bnds : xr.DataArray
        input wavelength bounds
    output_wavel_bnds : xr.DataArray
        output wavelength bounds
    weight_function : Callable, optional
        function used to determine weight of each input band, by default linear_weight
    weights : xr.DataArray, optional
        additional weights used for averaging, by default None

    Returns
    -------
    xr.Dataset
        data remapped to output_wavel_bnds
    """

    w = wavel_interpolation_weights(
        input_wavel_bnds,
        output_wavel_bnds,
        data.wavelength.values,
        output_wavel_bnds.wavelength.values,
        weight_function=weight_function,
    )

    if weights is None:
        ds = (data @ w).rename({"new_wavelength": "wavelength"})
    else:
        ds = (((data * weights) @ w) / (weights @ w)).rename({"new_wavelength": "wavelength"})

    return ds


def interpolate_ext(
    folder: Path,
    output_wavel_bnds: xr.DataArray,
    base_filename: str = BASE_FILENAME,
    weight_function=solar_planck_weights,
):
    """
    no additional weighting is required
    """
    file = folder / f"ext_{base_filename}"
    ext = xr.open_mfdataset(file)

    ds = interpolate_variable(
        data=ext.ext,
        input_wavel_bnds=ext.wavelength_bnds,
        output_wavel_bnds=output_wavel_bnds,
        weight_function=weight_function,
    )
    ds = xr.merge([ds.rename("ext"), ext[["time_bnds", "lat_bnds", "height_bnds"]]])
    ds.attrs = ext.attrs
    return ds


def interpolate_asy(
    folder,
    output_wavel_bnds: xr.DataArray,
    base_filename: str = BASE_FILENAME,
    weight_function=solar_planck_weights,
):
    """
    assymetry factor is weighted by scattering extinction
    """
    file = folder / f"ext_{base_filename}"
    ext = xr.open_mfdataset(file)

    file = folder / f"ssa_{base_filename}"
    ssa = xr.open_mfdataset(file)

    file = folder / f"asy_{base_filename}"
    asy = xr.open_mfdataset(file)

    weights = ext.ext * ssa.ssa
    ds = interpolate_variable(
        data=asy.asy,
        input_wavel_bnds=asy.wavelength_bnds,
        output_wavel_bnds=output_wavel_bnds,
        weight_function=weight_function,
        weights=weights,
    )
    ds = xr.merge([ds.rename("asy"), asy[["time_bnds", "lat_bnds", "height_bnds"]]])
    ds.attrs = asy.attrs
    return ds


def interpolate_ssa(
    folder,
    output_wavel_bnds: xr.DataArray,
    base_filename: str = BASE_FILENAME,
    weight_function=solar_planck_weights,
):
    """single scatter albedo is weighted by total extinction"""
    file = folder / f"ext_{base_filename}"
    ext = xr.open_mfdataset(file)

    file = folder / f"ssa_{base_filename}"
    ssa = xr.open_mfdataset(file)

    weights = ext.ext
    ds = interpolate_variable(
        data=ssa.ssa,
        input_wavel_bnds=ssa.wavelength_bnds,
        output_wavel_bnds=output_wavel_bnds,
        weight_function=weight_function,
        weights=weights,
    )
    ds = xr.merge([ds.rename("ssa"), ssa[["time_bnds", "lat_bnds", "height_bnds"]]])
    ds.attrs = ssa.attrs
    return ds
